import {EventEmitter, Input, OnInit, Output} from "@angular/core";
import {AbstractControl, FormGroup, ValidationErrors, Validators} from "@angular/forms";
import {isNullOrUndefined} from "@app/core/tools/is/is.tool";
import {AbstractComponent} from "@app/shared/components/abstract/abstract.component";
import {ModelAttributeEnum} from "@app/shared/enums/model-attribute.enum";

export abstract class AbstractFormComponent<T> extends AbstractComponent implements OnInit {
  public form: FormGroup;

  @Input()
  model: T;

  @Input()
  readonly: boolean;

  @Output()
  submitEvent: EventEmitter<T>;

  required: ValidationErrors = Validators.required;

  protected constructor() {
    super();
    this.submitEvent = new EventEmitter<T>();
  }

  ngOnInit(): void {
    if (this.model) {
      this.bindFormTo(this.model);
    } else {
      this.initNewForm();
    }
    this.disableIfReadonly();
  }

  private disableIfReadonly(): void {
    if (this.readonly) {
      this.form.disable();
    }
  }

  protected abstract initNewForm(): void;

  protected abstract bindFormTo(model: T): void;

  public onSubmit(): void {
    let model = this.form.value as T;
    model = this.treatmentBeforeSubmit(model);
    this.addResId(model);
    this.submitEvent.emit(model);
  }

  protected abstract treatmentBeforeSubmit(model: T): T;

  private addResId(model: T): void {
    const isUpdate = this.model !== null && this.model !== undefined;
    if (isUpdate) {
      model[ModelAttributeEnum.resId] = this.model[ModelAttributeEnum.resId];
    }
  }

  getFormControl(key: string): AbstractControl {
    return this.form.get(key);
  }

  isRequired(key: string): boolean {
    const errors = this.getFormControl(key).errors;
    return isNullOrUndefined(errors) ? false : errors.required;
  }
}
