import {ApiService} from "@app/core/http/api.service";
import {NotificationService} from "@app/core/services/notification.service";
import {Relation2TiersActionHelper} from "@app/core/stores/relation-2-tiers/relation-2-tiers-action.helper";
import {defaultResourceStateInitValue, ResourceState, ResourceStateModel} from "@app/core/stores/resource/resource.state";
import {StoreUtil} from "@app/core/utils/stores/store.util";
import {DepositStateModel} from "@app/deposit/stores/deposit.state";
import {DepositsModel, OrganizationalUnitsModel} from "@app/generated-api";
import {OverrideDefaultAction} from "@app/shared/decorators/store.decorator";
import {AccessResourceApiEnum, AdminResourceApiEnum, ResourceApiEnum} from "@app/shared/enums/api.enum";
import {ModelAttributeEnum} from "@app/shared/enums/model-attribute.enum";
import {StateEnum} from "@app/shared/enums/state.enum";
import {CollectionTypedModel} from "@app/shared/models/collection-typed.model";
import {StateModel} from "@app/shared/models/state.model";
import {OrgUnitAction, orgUnitActionNameSpace} from "@app/shared/stores/organizational-unit/organizational-unit.action";
import {orgUnitSubmissionPolicyActionNameSpace} from "@app/shared/stores/organizational-unit/submission-policy/organizational-unit-submission-policy.action";
import {
  OrganizationalUnitSubmissionPolicyState,
  OrganizationalUnitSubmissionPolicyStateModel,
} from "@app/shared/stores/organizational-unit/submission-policy/organizational-unit-submission-policy.state";
import {PreservationPolicyAction} from "@app/shared/stores/preservation-policy/preservation-policy.action";
import {SubmissionPolicyAction} from "@app/shared/stores/submission-policy/submission-policy.action";
import {Action, Actions, ofActionCompleted, State, StateContext, Store} from "@ngxs/store";

import _ from "lodash";
import {Observable} from "rxjs";
import {catchError} from "rxjs/internal/operators/catchError";
import {tap} from "rxjs/internal/operators/tap";
import {ResourceNameSpace} from "../../../core/stores/resource/resource-namespace.model";

export interface OrganizationalUnitStateModel extends ResourceStateModel<OrganizationalUnitsModel> {
  [StateEnum.organizationalUnitSubResourceSubmissionPolicy]?: OrganizationalUnitSubmissionPolicyStateModel;
}

@State<OrganizationalUnitStateModel>({
  name: StateEnum.organizationalUnit,
  defaults: {
    ...defaultResourceStateInitValue,
  },
  children: [
    OrganizationalUnitSubmissionPolicyState,
  ],
})
export class OrganizationalUnitState extends ResourceState<OrganizationalUnitsModel> {

  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService);
  }

  defineUrlResource(): ResourceApiEnum {
    const isLoggedId = this.store.selectSnapshot((state: StateModel) => state[StateEnum.application].isLoggedIn);
    if (isLoggedId) {
      return AdminResourceApiEnum.organizationalUnits;
    }
    return AccessResourceApiEnum.organizationalUnits;
  }

  defineResourceNameSpace(): ResourceNameSpace {
    return orgUnitActionNameSpace;
  }

  defineState(): StateEnum {
    return StateEnum.organizationalUnit;
  }

  @Action(OrgUnitAction.LoadResource)
  loadResource(ctx: StateContext<DepositStateModel>, action: OrgUnitAction.LoadResource): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });

    ctx.dispatch([
      new PreservationPolicyAction.GetAll(),
      new SubmissionPolicyAction.GetAll(),
      new OrgUnitAction.LoadResourceSuccess(action),
    ]);
  }

  @Action(OrgUnitAction.LoadResourceSuccess)
  loadResourceSuccess(ctx: StateContext<DepositStateModel>, action: OrgUnitAction.LoadResourceSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
  }

  @Action(OrgUnitAction.LoadResourceFail)
  loadResourceFail(ctx: StateContext<DepositStateModel>, action: OrgUnitAction.LoadResourceFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
  }

  @Action(OrgUnitAction.GetAllFromAccess)
  getAllAccess(ctx: StateContext<DepositStateModel>, action: OrgUnitAction.GetAllFromAccess): Observable<CollectionTypedModel<DepositsModel>> {
    // TOFIX Find a better way to manage get all Admin / get all Access (different store ?)
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
      queryParameters: StoreUtil.getQueryParametersToApply(action.queryParameters, ctx),
    });
    return this.apiService.get<DepositsModel>(AccessResourceApiEnum.organizationalUnits, ctx.getState().queryParameters)
      .pipe(
        tap((collection: CollectionTypedModel<DepositsModel>) => {
          ctx.dispatch(new OrgUnitAction.GetAllSuccess(action, collection));
        }),
        catchError(error => {
          ctx.dispatch(new OrgUnitAction.GetAllFail(action));
          throw error;
        }),
      );
  }

  @OverrideDefaultAction()
  @Action(OrgUnitAction.Create)
  create(ctx: StateContext<OrganizationalUnitStateModel>, action: OrgUnitAction.Create): Observable<OrganizationalUnitsModel> {
    return super.internalCreate(ctx, action)
      .pipe(
        tap(model => {
          const newSubmissionPolicies = action.model[ModelAttributeEnum.submissionPolicies];

          StoreUtil.dispatchActionAndWaitForSubActionsCompletion(
            ctx,
            [
              {
                action: Relation2TiersActionHelper.updateSubResource(orgUnitSubmissionPolicyActionNameSpace, model.resId, [], newSubmissionPolicies),
                subActionCompletions: [
                  this.actions$.pipe(ofActionCompleted(orgUnitSubmissionPolicyActionNameSpace.UpdateSuccess)),
                  this.actions$.pipe(ofActionCompleted(orgUnitSubmissionPolicyActionNameSpace.UpdateFail)),
                ],
              },
            ],
          ).subscribe(success => {
            if (success) {
              ctx.dispatch(new orgUnitActionNameSpace.CreateSuccess());
            } else {
              ctx.dispatch(new orgUnitActionNameSpace.CreateFail());
            }
          });
        }),
      );
  }

  @OverrideDefaultAction()
  @Action(OrgUnitAction.Update)
  update(ctx: StateContext<OrganizationalUnitStateModel>, action: OrgUnitAction.Update): Observable<OrganizationalUnitsModel> {
    return super.internalUpdate(ctx, action)
      .pipe(
        tap(model => {
          const oldSubmissionPolicies = _.map(ctx.getState()[StateEnum.organizationalUnitSubResourceSubmissionPolicy].selected, ModelAttributeEnum.resId);
          const newSubmissionPolicies = action.model[ModelAttributeEnum.submissionPolicies];

          StoreUtil.dispatchActionAndWaitForSubActionsCompletion(
            ctx,
            [
              {
                action: Relation2TiersActionHelper.updateSubResource(orgUnitSubmissionPolicyActionNameSpace, action.model.resId, oldSubmissionPolicies, newSubmissionPolicies),
                subActionCompletions: [
                  this.actions$.pipe(ofActionCompleted(orgUnitSubmissionPolicyActionNameSpace.UpdateSuccess)),
                  this.actions$.pipe(ofActionCompleted(orgUnitSubmissionPolicyActionNameSpace.UpdateFail)),
                ],
              },
            ],
          ).subscribe(success => {
            if (success) {
              ctx.dispatch(new orgUnitActionNameSpace.UpdateSuccess());
            } else {
              ctx.dispatch(new orgUnitActionNameSpace.UpdateFail());
            }
          });
        }),
      );
  }
}
